@@ -80,16 +80,16 @@ class AlignmentStatistics():
             # Count how many precursors / peptides / proteins fall below the threshold
             if m.find_best_peptide_pg().get_fdr_score() < fdr_cutoff:
                 astats.nr_good_precursors += 1
-                astats.good_peptides.update([m.get_peptides()[0].sequence])
-                astats.good_proteins.update([m.get_peptides()[0].protein_name])
+                astats.good_peptides.update([m.getAllPeptides()[0].sequence])
+                astats.good_proteins.update([m.getAllPeptides()[0].protein_name])
 
             # Count how many precursors / peptides / proteins were quantified
             if len(m.get_selected_peakgroups()) > 0:
                 astats.nr_quant_precursors += 1
-                astats.quant_peptides.update([m.get_peptides()[0].sequence])
-                astats.quant_proteins.update([m.get_peptides()[0].protein_name])
+                astats.quant_peptides.update([m.getAllPeptides()[0].sequence])
+                astats.quant_proteins.update([m.getAllPeptides()[0].protein_name])
 
-            for p in m.get_peptides():
+            for p in m.getAllPeptides():
 
                 # Count how many peakgroups simply fall below the threshold
                 if p.get_best_peakgroup().get_fdr_score() < fdr_cutoff:
@@ -253,7 +253,7 @@ class Experiment(MRExperiment):
             if (len(selected_peakgroups)*1.0 / len(self.runs)) < fraction_needed_selected: 
                 continue
 
-            for p in m.get_peptides():
+            for p in m.getAllPeptides():
                 selected_pg = p.get_selected_peakgroup()
                 clustered_pg = p.getClusteredPeakgroups()
                 for pg in clustered_pg:
@@ -305,7 +305,12 @@ class Experiment(MRExperiment):
                     row_to_write[ header_dict["run_id"]] = selected_ids_dict[f_id].peptide.run.get_id()
                     writer.writerow(row_to_write)
 
-        elif len(outfile) > 0 and file_format == "openswath":
+        elif len(outfile) > 0 and file_format in ["openswath", "peakview_preprocess"]:
+
+            name_of_id_col_map = { "openswath" : "id" , "peakview_preprocess" : "preprocess_id"}
+            name_of_trgr_col_map = { "openswath" : "transition_group_id" , "peakview_preprocess" : "Pep Index"}
+            name_of_id_col = name_of_id_col_map[file_format]
+            name_of_trgr_col = name_of_trgr_col_map[file_format]
 
             # Only in openswath we have the ID and can go back to the original file.
             # We can write out the complete original files.
@@ -331,18 +336,19 @@ class Experiment(MRExperiment):
                 header_dict[n] = i
 
               for row in reader:
-                  f_id = row[ header_dict["id"]]
+                  f_id = row[ header_dict[name_of_id_col]]
                   if selected_ids_dict.has_key(f_id):
                       # Check the "id" and "transition_group_id" field.
                       # Unfortunately the id can be non-unique, there we check both.
                       trgroup_id = selected_ids_dict[f_id].peptide.get_id()
-                      unique_peptide_id = row[ header_dict["transition_group_id"]]
+                      unique_peptide_id = row[ header_dict[name_of_trgr_col]]
                       if unique_peptide_id == trgroup_id:
                           row_to_write = row
                           row_to_write += [selected_ids_dict[f_id].peptide.run.get_id(), f, selected_ids_dict[f_id].get_cluster_id()]
                           # Replace run_id with the aligned id (align_runid) ->
                           # otherwise the run_id is not guaranteed to be unique 
-                          row_to_write[ header_dict["run_id"]] = selected_ids_dict[f_id].peptide.run.get_id()
+                          if file_format == "openswath" : 
+                              row_to_write[ header_dict["run_id"]] = selected_ids_dict[f_id].peptide.run.get_id()
                           writer.writerow(row_to_write)
 
         # 5. Write out the .tr transformation files
@@ -402,7 +408,7 @@ def estimate_aligned_fdr_cutoff(options, this_exp, multipeptides, fdr_range):
                     p.unselect_all()
             return aligned_fdr_cutoff
 
-def doMSTAlignment(exp, multipeptides, max_rt_diff, initial_alignment_cutoff,
+def doMSTAlignment(exp, multipeptides, max_rt_diff, rt_diff_isotope, initial_alignment_cutoff,
                    fdr_cutoff, aligned_fdr_cutoff, smoothing_method, method,
                    use_RT_correction, stdev_max_rt_per_run, use_local_stdev):
     """
@@ -423,7 +429,8 @@ def doMSTAlignment(exp, multipeptides, max_rt_diff, initial_alignment_cutoff,
     tree_mapped = [ (exp.runs[a].get_id(), exp.runs[b].get_id()) for a,b in tree]
 
     # Perform work
-    al = TreeConsensusAlignment(max_rt_diff, fdr_cutoff, aligned_fdr_cutoff,
+    al = TreeConsensusAlignment(max_rt_diff, fdr_cutoff, aligned_fdr_cutoff, 
+                                rt_diff_isotope=rt_diff_isotope,
                                 correctRT_using_pg=use_RT_correction,
                                 stdev_max_rt_per_run=stdev_max_rt_per_run,
                                 use_local_stdev=use_local_stdev)
@@ -540,17 +547,19 @@ def handle_args():
     parser.add_argument("--out_matrix", dest="matrix_outfile", default="", help="Matrix containing one peak group per row (supports .csv, .tsv or .xlsx)")
     parser.add_argument("--out_ids", dest="ids_outfile", default="", help="Id file only containing the ids")
     parser.add_argument("--out_meta", dest="yaml_outfile", default="", help="Outfile containing meta information, e.g. mapping of runs to original directories")
-    parser.add_argument("--fdr_cutoff", dest="fdr_cutoff", default=0.01, type=float, help="FDR cutoff to use, default 0.01", metavar='0.01')
+    parser.add_argument("--fdr_cutoff", dest="fdr_cutoff", default=0.01, type=float, help="Seeding score cutoff", metavar='0.01')
+    parser.add_argument("--max_fdr_quality", dest="aligned_fdr_cutoff", default=-1.0, help="Extension score cutoff - during the extension phase of the algorithm, peakgroups of this quality will still be considered for alignment (in FDR) - it is possible to give a range in the format lower,higher+stepsize,stepsize - e.g. 0,0.31,0.01 (-1 will set it to fdr_cutoff)", metavar='-1')
     parser.add_argument("--max_rt_diff", dest="rt_diff_cutoff", default=30, help="Maximal difference in RT for two aligned features", metavar='30')
-    parser.add_argument("--max_fdr_quality", dest="aligned_fdr_cutoff", default=-1.0, help="Quality cutoff to still consider a feature for alignment (in FDR) - it is possible to give a range in the format lower,higher+stepsize,stepsize - e.g. 0,0.31,0.01 (-1 will set it to fdr_cutoff)", metavar='-1')
+    parser.add_argument("--iso_max_rt_diff", dest="rt_diff_isotope", default=10, help="Maximal difference in RT for two isotopic channels in the same run", metavar='30')
     parser.add_argument("--frac_selected", dest="min_frac_selected", default=0.0, type=float, help="Do not write peakgroup if selected in less than this fraction of runs (range 0 to 1)", metavar='0')
     parser.add_argument('--method', default='best_overall', help="Which method to use for the clustering (best_overall, best_cluster_score or global_best_cluster_score, global_best_overall, LocalMST, LocalMSTAllCluster). Note that the MST options will perform a local, MST guided alignment while the other options will use a reference-guided alignment. The global option will also move peaks which are below the selected FDR threshold.")
     parser.add_argument("--verbosity", default=0, type=int, help="Verbosity (0 = little)", metavar='0')
     parser.add_argument("--matrix_output_method", dest="matrix_output_method", default='none', help="Which columns are written besides Intensity (none, RT, score, source or full)")
-    parser.add_argument('--realign_method', dest='realign_method', default="diRT", help="How to re-align runs in retention time ('diRT': use only deltaiRT from the input file, 'linear': perform a linear regression using best peakgroups, 'splineR': perform a spline fit using R, 'splineR_external': perform a spline fit using R (start an R process using the command line, 'splinePy' use Python native spline from scikits.datasmooth (slow!), 'lowess': use Robust locally weighted regression (lowess smoother)")
+    parser.add_argument('--realign_method', dest='realign_method', default="diRT", help="How to re-align runs in retention time ('diRT': use only deltaiRT from the input file, 'linear': perform a linear regression using best peakgroups, 'splineR': perform a spline fit using R, 'splineR_external': perform a spline fit using R (start an R process using the command line, 'splinePy' use Python native spline from scikits.datasmooth (slow!), 'lowess': use Robust locally weighted regression (lowess smoother), 'nonCVSpline, CVSpline': splines with and without cross-validation, 'earth' : use Multivariate Adaptive Regression Splines using py-earth")
 
     experimental_parser = parser.add_argument_group('experimental options')
 
+    experimental_parser.add_argument('--disable_isotopic_grouping', action='store_true', default=False, help="Disable grouping of isotopic variants by peptide_group_label, thus disabling matching of isotopic variants of the same peptide across channels. If turned off, each isotopic channel will be matched independently of the other. If enabled, the more certain identification will be used to infer the location of the peak in the other channel.")
     experimental_parser.add_argument('--use_dscore_filter', action='store_true', default=False)
     experimental_parser.add_argument("--dscore_cutoff", default=1.96, type=float, help="Quality cutoff to still consider a feature for alignment using the d_score: everything below this d-score is discarded", metavar='1.96')
     experimental_parser.add_argument("--nr_high_conf_exp", default=1, type=int, help="Number of experiments in which the peptide needs to be identified with high confidence (e.g. above fdr_curoff)", metavar='1')
@@ -611,7 +620,9 @@ def main(options):
 
     # Read the files
     start = time.time()
-    reader = SWATHScoringReader.newReader(options.infiles, options.file_format, options.readmethod, readfilter)
+    reader = SWATHScoringReader.newReader(options.infiles, options.file_format,
+                                          options.readmethod, readfilter,
+                                          enable_isotopic_grouping = not options.disable_isotopic_grouping)
     runs = reader.parse_files(options.realign_method != "diRT", options.verbosity)
 
     # Create experiment
@@ -635,7 +646,8 @@ def main(options):
         else:
             stdev_max_rt_per_run = None
             
-        doMSTAlignment(this_exp, multipeptides, float(options.rt_diff_cutoff),
+        doMSTAlignment(this_exp, multipeptides, float(options.rt_diff_cutoff), 
+                       float(options.rt_diff_isotope),
                        float(options.alignment_score), options.fdr_cutoff,
                        float(options.aligned_fdr_cutoff),
                        options.realign_method, options.method,
@@ -654,7 +666,7 @@ def main(options):
             if pg.get_fdr_score() < options.fdr_cutoff:
                 count += 1
         if count < options.nr_high_conf_exp:
-            for p in mpep.get_peptides():
+            for p in mpep.getAllPeptides():
                 p.unselect_all()
 
     # print statistics, write output
