@@ -39,6 +39,8 @@ import os, sys, csv, time
 import numpy
 import argparse
 from msproteomicstoolslib.format.SWATHScoringReader import *
+from msproteomicstoolslib.data_structures.Precursor import GeneralPrecursor, Precursor
+from msproteomicstoolslib.data_structures.PrecursorGroup import PrecursorGroup
 import msproteomicstoolslib.format.TransformationCollection as transformations
 from msproteomicstoolslib.algorithms.alignment.Multipeptide import Multipeptide
 from msproteomicstoolslib.algorithms.alignment.SplineAligner import SplineAligner
@@ -194,8 +196,8 @@ class SwathChromatogramCollection(object):
         return self.allruns.keys()
 
 # Main entry points:
-# runSingleFileImputation
-# runImputeValues
+# runSingleFileImputation -> for tree based, new imputation
+# runImputeValues -> old method using .tr files
 
 def runSingleFileImputation(options, peakgroups_file, mzML_file, method):
     """Impute values across chromatograms
@@ -219,7 +221,10 @@ def runSingleFileImputation(options, peakgroups_file, mzML_file, method):
     fdr_cutoff_all_pg = 1.0
 
     start = time.time()
-    reader = SWATHScoringReader.newReader([peakgroups_file], options.file_format, readmethod="complete")
+    reader = SWATHScoringReader.newReader([peakgroups_file],
+                                          options.file_format,
+                                          readmethod="complete",
+                                          enable_isotopic_grouping = not options.disable_isotopic_grouping)
     new_exp = Experiment()
     new_exp.runs = reader.parse_files()
     multipeptides = new_exp.get_all_multipeptides(fdr_cutoff_all_pg, verbose=False)
@@ -228,7 +233,8 @@ def runSingleFileImputation(options, peakgroups_file, mzML_file, method):
     mapping = {}
     precursors_mapping = {}
     sequences_mapping = {}
-    inferMapping([ mzML_file ], [ peakgroups_file ], mapping, precursors_mapping, sequences_mapping, verbose=False)
+    protein_mapping = {}
+    inferMapping([ mzML_file ], [ peakgroups_file ], mapping, precursors_mapping, sequences_mapping, protein_mapping, verbose=False)
     mapping_inv = dict([(v[0],k) for k,v in mapping.iteritems()])
     if VERBOSE:
         print mapping
@@ -279,7 +285,8 @@ def runSingleFileImputation(options, peakgroups_file, mzML_file, method):
     print("Alignment took %ss" % (time.time() - start) )
     start = time.time()
     multipeptides = analyze_multipeptides(new_exp, multipeptides, swath_chromatograms,
-        tr_data, options.border_option, rid, tree=tree_mapped, mat=dist_matrix)
+        tr_data, options.border_option, rid, tree=tree_mapped, mat=dist_matrix,
+        disable_isotopic_transfer=options.disable_isotopic_transfer)
     print("Analyzing the runs took %ss" % (time.time() - start) )
 
     return new_exp, multipeptides
@@ -308,7 +315,10 @@ def runImputeValues(options, peakgroups_file, trafo_fnames):
     fdr_cutoff_all_pg = 1.0
 
     start = time.time()
-    reader = SWATHScoringReader.newReader([peakgroups_file], options.file_format, readmethod="complete")
+    reader = SWATHScoringReader.newReader([peakgroups_file],
+                                          options.file_format,
+                                          readmethod="complete",
+                                          enable_isotopic_grouping = not options.disable_isotopic_grouping)
     new_exp = Experiment()
     new_exp.runs = reader.parse_files()
     multipeptides = new_exp.get_all_multipeptides(fdr_cutoff_all_pg, verbose=False)
@@ -336,7 +346,8 @@ def runImputeValues(options, peakgroups_file, trafo_fnames):
         # 
         start = time.time()
         multipeptides = analyze_multipeptides(new_exp, multipeptides, swath_chromatograms,
-            transformation_collection_, options.border_option, rid)
+            transformation_collection_, options.border_option, 
+            onlyExtractFromRun=rid, disable_isotopic_transfer=options.disable_isotopic_transfer)
         print("Analyzing the runs took %ss" % (time.time() - start) )
         return new_exp, multipeptides
 
@@ -357,22 +368,38 @@ def runImputeValues(options, peakgroups_file, trafo_fnames):
             # Create the cache for run "rid" and then only extract peakgroups from this run
             swath_chromatograms.createRunCache(rid)
             multipeptides = analyze_multipeptides(new_exp, multipeptides, 
-                swath_chromatograms, transformation_collection_, options.border_option, rid)
+                swath_chromatograms, transformation_collection_, options.border_option, 
+                onlyExtractFromRun=rid, disable_isotopic_transfer=options.disable_isotopic_transfer)
     else:
         multipeptides = analyze_multipeptides(new_exp, multipeptides, swath_chromatograms,
-            transformation_collection_, options.border_option)
+            transformation_collection_, options.border_option, disable_isotopic_transfer=options.disable_isotopic_transfer)
     print("Analyzing the runs took %ss" % (time.time() - start) )
     return new_exp, multipeptides
 
 def analyze_multipeptides(new_exp, multipeptides, swath_chromatograms, 
-                          transformation_collection_, border_option, onlyExtractFromRun=None, tree=None, mat=None):
+                          transformation_collection_, border_option,
+                          onlyExtractFromRun=None, tree=None, mat=None,
+                          disable_isotopic_transfer=False):
     """Analyze the multipeptides and impute missing values
 
+    This function has three different modes:
+
+        - if a tree is given, it will use the single shortest path on the tree to infer the boundaries
+        - if a distance matrix is given, it will use the closest run overall to infer the boundaries
+        - if neither is given, it will use a mean / median / maximum of all
+          borders by converting all boundaries to the frame of the reference run
+          and then to the current run
+
     Args:
         new_exp(AlignmentExperiment): experiment containing the aligned peakgroups
         multipeptides(list(AlignmentHelper.Multipeptide)): list of multipeptides
         swath_chromatograms(dict): containing the objects pointing to the original chrom mzML (see runImputeValues)
         transformation_collection_(.TransformationCollection): specifying how to transform between retention times of different runs
+        border_option(String): one of the following options ("mean", "median" "max_width"), determining how to aggregate multiple peak boundary information
+        onlyExtractFromRun(String): Whether only to perform signal extraction from a single run (if not None, needs to be a run id)
+        tree(MinimumSpanningTree): alignment guidance tree (if given, will use shortest path approach)
+        mat( matrix(float) ): distance matrix, see getDistanceMatrix (if given, will use closest run approach)
+        disable_isotopic_transfer(bool): whether to use isotopic grouping (e.g. group heavy/light channels together)
 
     Returns:
         The updated multipeptides
@@ -393,25 +420,86 @@ def analyze_multipeptides(new_exp, multipeptides, swath_chromatograms,
         if i % 500 == 0: 
             print "Done with %s out of %s" % (i, len(multipeptides))
         if VERBOSE:
-            print "do", m.get_peptides()[0].get_id()
+            print "Do group of peptides", m.getPrecursorGroups()[0].getPeptideGroupLabel()
 
-        # Ensure that each run has either exactly one peakgroup (or zero)
-        if all ([pg.get_cluster_id() == -1 for p in m.get_peptides() for pg in p.get_all_peakgroups()]) and \
-           any([len(p.peakgroups) > 1 for p in m.get_peptides()] ):
-            raise Exception("Found more than one peakgroup for peptide %s - \
-                \n is this after alignment or did you forget to run feature_alignment.py beforehand?" % (
-                m.get_id() ))
 
-        clusters = set( [pg.get_cluster_id() for p in m.get_peptides() for pg in p.get_all_peakgroups()])
+        # Ensure that each run has either exactly one peakgroup (or zero):
+        # If all peakgroups are old-style peakgroups (only single cluster) then
+        # all peptides can only have a single peakgroup.
+        if all([pg.get_cluster_id() == -1 for p in m.getAllPeptides() for pg in p.get_all_peakgroups()]):
+            if any([len(p.peakgroups) > 1 for p in m.getAllPeptides()] ):
+                raise Exception("Found more than one peakgroup for peptide %s - \
+                    \n is this after alignment or did you forget to run feature_alignment.py beforehand?" % (
+                    m.get_id() ))
+
+        clusters = set( [pg.get_cluster_id() for p in m.getAllPeptides() for pg in p.get_all_peakgroups()])
+        if VERBOSE: print "Shall work on clusters ", clusters
         for cl in clusters:
-            selected_pg = [pg for p in m.get_peptides() for pg in p.get_all_peakgroups() if pg.get_cluster_id() == cl]
-            analyze_multipeptide_cluster(m, cnt, new_exp, swath_chromatograms, 
-                                      transformation_collection_, border_option, selected_pg, cl,
-                                      onlyExtractFromRun, tree, mat)
 
-        if all ([pg.get_cluster_id() == -1 for p in m.get_peptides() for pg in p.get_all_peakgroups()]):
+            # Retrieve all transition group ids available for this precursor group
+            # Iterate through all ids (in order of ascending FDR since we
+            # believe transferring borders between runs is most efficient when
+            # first using the group with the best overall score).
+            trgr_ids = set([ trgr.get_id() for prgr in m.getPrecursorGroups() for trgr in prgr ])
+            fdr_trgr_mapping = []
+            for trgr_id in trgr_ids:
+                selected_pg = [pg.get_fdr_score() for p in m.getAllPeptides() for pg in p.get_all_peakgroups()
+                    if pg.peptide.get_id() == trgr_id and pg.get_cluster_id() == cl]
+
+                # In some cases there is no peakgroup for the current cluster available
+                if len(selected_pg) > 0:
+                    fdr_trgr_mapping.append( (min(selected_pg), trgr_id) )
+
+            if VERBOSE: print "mapping", sorted(fdr_trgr_mapping), "vs", trgr_ids
+            for fdr, trgr_id in sorted(fdr_trgr_mapping):
+                if VERBOSE: print "================================================"
+                if VERBOSE: print "Shall work on transition group", trgr_id, "with cluster", cl
+
+                # Get all selected peakgroups that correspond to the current transition group id 
+                selected_pg = [pg for p in m.getAllPeptides() for pg in p.get_all_peakgroups()
+                    if pg.peptide.get_id() == trgr_id and pg.get_cluster_id() == cl]
+                analyze_multipeptide_cluster(m, cnt, new_exp, swath_chromatograms, 
+                                          transformation_collection_, border_option, selected_pg, cl,
+                                          onlyExtractFromRun, tree, mat)
+
+
+            # Ensure consistency in peak picking between isotopic channels by
+            # looking at the best channel for each run and then forcing the
+            # peak boundaries of this channel unto the other channels. This is
+            # done even if a very good peak was found in the other channels,
+            # but ensuring that the peak boundaries are the same is more
+            # important here.
+            # disable_isotopic_transfer turns this feature off
+            if disable_isotopic_transfer:
+                continue
+
+            for rid in [r.get_id() for r in new_exp.runs]:
+                if m.hasPrecursorGroup(rid):
+                    prgr = m.getPrecursorGroup(rid)
+                    pgs = [(pg_.get_fdr_score(), pg_) for prec_ in prgr for pg_ in prec_.peakgroups if pg_.get_cluster_id() == cl] 
+                    if len(pgs) > 0:
+                        best_pg = min(pgs)[1]
+                        border_l = float(best_pg.get_value("leftWidth"))
+                        border_r = float(best_pg.get_value("rightWidth"))
+                        for prec in m.getPrecursorGroup(rid):
+                            currpg = [pg for pg in prec.get_all_peakgroups() if pg.get_cluster_id() == cl]
+                            if len(currpg) == 1 and currpg[0].get_feature_id() != best_pg.get_feature_id():
+                                pg = currpg[0]
+                                # Propagate peak boundaries of the best peakgroup to the current one
+                                current_run = [r for r in new_exp.runs if r.get_id() == rid][0]
+                                newpg = integrate_chromatogram(pg, current_run, swath_chromatograms,
+                                                             border_l, border_r, cnt)
+                                pg.set_value("Intensity", newpg.get_value("Intensity"))
+                                pg.set_intensity(newpg.get_intensity())
+                                pg.set_value("leftWidth", newpg.get_value("leftWidth"))
+                                pg.set_value("rightWidth", newpg.get_value("rightWidth"))
+                                pg.set_value("RT", newpg.get_value("RT"))
+                                pg.set_value("m_score", 1.5)
+
+
+        if all ([pg.get_cluster_id() == -1 for p in m.getAllPeptides() for pg in p.get_all_peakgroups()]):
             # no cluster info was read in -> select all
-            for p in m.get_peptides():
+            for p in m.getAllPeptides():
                 for pg in p.get_all_peakgroups():
                     pg.select_this_peakgroup()
 
@@ -425,6 +513,7 @@ def analyze_multipeptide_cluster(current_mpep, cnt, new_exp, swath_chromatograms
                           onlyExtractFromRun=None, tree=None, mat=None):
 
         for rid in [r.get_id() for r in new_exp.runs]:
+            if VERBOSE: print "Shall work on run ", rid
             cnt.peakgroups += 1
 
             # Check whether we have a peakgroup for this run id, if yes we mark
@@ -442,28 +531,87 @@ def analyze_multipeptide_cluster(current_mpep, cnt, new_exp, swath_chromatograms
                 current_run = [r for r in new_exp.runs if r.get_id() == rid][0]
                 rmap = dict([(r.get_id(),i) for i,r in enumerate(new_exp.runs) ])
 
-                if VERBOSE: 
-                    print "Will try to fill NA in run", current_run.get_id(), "for peptide", current_mpep.get_peptides()[0].get_id()
-                if tree is not None:
+                if VERBOSE:
+                    print "Will try to fill NA in run", current_run.get_id(), "for peptide", selected_pg[0].peptide.get_id()
+                    print tree, mat
+
+                inferred_from_isotope = False
+                if current_mpep.hasPrecursorGroup(current_run.get_id()):
+                    # If another precursor was already picked for this run,
+                    # check if any peakgroups of the same cluster exist and if
+                    # so, try to use the one with the minimal FDR score to
+                    # infer the boundaries.
+                    prgr = current_mpep.getPrecursorGroup(current_run.get_id())
+                    pgs = [(pg_.get_fdr_score(), pg_) for prec_ in prgr for pg_ in prec_.peakgroups if pg_.get_cluster_id() == cluster_id] 
+                    if len(pgs) > 0:
+                        # TODO if the boundaries were inferred (e.g. fdr score > 1.0), should we not use them? 
+                        best_pg = min(pgs)[1]
+                        border_l = float(best_pg.get_value("leftWidth"))
+                        border_r = float(best_pg.get_value("rightWidth"))
+                        inferred_from_isotope = True
+
+                        if VERBOSE:
+                            print "Will try to infer from isotopically modified version of the same run!"
+                            print "Precursor is", prgr
+                            for prec in prgr:
+                                print " --", prec
+                                for pg in prec.peakgroups:
+                                    print "  * ", pg
+                            print "Min fdr pg", best_pg, " border %s / %s" % (border_l, border_r)
+
+                if inferred_from_isotope:
+                    # All good
+                    pass
+
+                elif tree is not None:
+                    ## Use the closest path approach
                     border_l, border_r = integrationBorderShortestPath(selected_pg, 
                         rid, transformation_collection_, tree)
                 elif mat is not None:
+                    ## Use the closest overall run approach
                     border_l, border_r = integrationBorderShortestDistance(selected_pg, 
                         rid, transformation_collection_, mat, rmap)
                 else:
+                    ## Use the refernce-based approach
                     border_l, border_r = integrationBorderReference(new_exp, selected_pg, rid, transformation_collection_, border_option)
                 newpg = integrate_chromatogram(selected_pg[0], current_run, swath_chromatograms,
                                              border_l, border_r, cnt)
                 if newpg != "NA": 
                     if VERBOSE: 
                         print "Managed to fill NA in run", current_run.get_id(), \
-                          "with value", newpg.get_value("Intensity"), "/ borders", border_l, border_r
+                          "with value", newpg.get_value("Intensity"), "/ borders", border_l, border_r #, "for cluster", newpg.get_value("align_clusterid")
                     cnt.imputation_succ += 1
-                    precursor = GeneralPrecursor(newpg.get_value("transition_group_id"), current_run)
-                    # Select for output, add to precursor
+                    transition_group_id = newpg.get_value("transition_group_id")
+                    try:
+                        peptide_label = newpg.get_value("peptide_group_label")
+                    except KeyError:
+                        peptide_label = transition_group_id
+
+                    # Create new precursor
+                    precursor = GeneralPrecursor(transition_group_id, current_run)
                     newpg.setClusterID(cluster_id)
+                    newpg.peptide = precursor
                     precursor.add_peakgroup(newpg)
-                    current_mpep.insert(rid, precursor)
+                    precursor.sequence = selected_pg[0].peptide.sequence
+                    precursor.protein_name = selected_pg[0].peptide.protein_name
+
+                    if current_mpep.hasPrecursorGroup(current_run.get_id()):
+                        prec_group = current_mpep.getPrecursorGroup(current_run.get_id())
+                        if prec_group.getPrecursor(transition_group_id) is None:
+                            # No precursors exists yet for this transition_group_id - this
+                            # means that we have a new run for which a precursor group already
+                            # exists (e.g. we have found a light version already) and are now
+                            # dealing with the heavy ...
+                            prec_group.addPrecursor(precursor)
+                        else:
+                            # Likely, another peakgroup from the same run and peptide was
+                            # picked already but not from the same cluster
+                            prec_group.getPrecursor(transition_group_id).add_peakgroup(newpg)
+                    else:
+                        # Create new precursor group and insert
+                        precursor_group = PrecursorGroup(peptide_label, current_run)
+                        precursor_group.addPrecursor(precursor)
+                        current_mpep.insert(rid, precursor_group)
 
 def integrate_chromatogram(template_pg, current_run, swath_chromatograms, 
                            left_start, right_end, cnt):
@@ -488,10 +636,10 @@ def integrate_chromatogram(template_pg, current_run, swath_chromatograms,
 
     # Create new peakgroup by copying the old one
     newrow = ["NA" for ele in template_pg.row]
-    newpg = GeneralPeakGroup(newrow, current_run, template_pg.peptide)
+    newpg = GeneralPeakGroup(newrow, current_run, None)
     for element in ["transition_group_id"]:
         newpg.set_value(element, template_pg.get_value(element))
-    for element in ["decoy", "Sequence", "FullPeptideName", "Charge", "ProteinName", "nr_peaks", "m.z", "m/z", "align_clusterid"]:
+    for element in ["decoy", "Sequence", "FullPeptideName", "Charge", "ProteinName", "nr_peaks", "m.z", "m/z", "align_clusterid", "peptide_group_label"]:
         try:
             newpg.set_value(element, template_pg.get_value(element))
         except KeyError:
@@ -512,7 +660,7 @@ def integrate_chromatogram(template_pg, current_run, swath_chromatograms,
     import uuid 
     thisid = str(uuid.uuid1() )
     newpg.set_normalized_retentiontime((left_start + right_end) / 2.0 )
-    newpg.set_fdr_score(1.0)
+    newpg.set_fdr_score(2.0)
     newpg.set_feature_id(thisid)
 
     integrated_sum = 0
@@ -562,7 +710,7 @@ def write_out(new_exp, multipeptides, outfile, matrix_outfile, single_outfile):
     for m in multipeptides:
         # selected_peakgroups = [p.peakgroups[0] for p in m.get_peptides()]
         # if (len(selected_peakgroups)*2.0 / len(new_exp.runs) < fraction_needed_selected) : continue
-        for p in m.get_peptides():
+        for p in m.getAllPeptides():
             for selected_pg in p.peakgroups:
                 if single_outfile:
                     # Only write the newly imputed ones ... 
@@ -596,18 +744,23 @@ def handle_args():
     parser.add_argument('--do_single_run', default='', metavar="", help="Only do a single run")
 
     experimental_parser = parser.add_argument_group('experimental options')
+    experimental_parser.add_argument('--disable_isotopic_grouping', action='store_true', default=False, help="Disable grouping of isotopic variants by peptide_group_label, thus disabling matching of isotopic variants of the same peptide across channels. If turned off, each isotopic channel will be matched independently of the other. If enabled, the more certain identification will be used to infer the location of the peak in the other channel.")
+    experimental_parser.add_argument('--disable_isotopic_transfer', action='store_true', default=False, help="Disable the transfer of isotopic boundaries in all cases. If enabled (default), the best (best score) isotopic channel dictates the peak boundaries and all other channels use those boundaries. This ensures consistency in peak picking in all cases.")
 
     args = parser.parse_args(sys.argv[1:])
     args.verbosity = int(args.verbosity)
     return args
 
 def main(options):
-    import time
+
     if options.method in ["singleShortestPath", "singleClosestRun"]:
         new_exp, multipeptides = runSingleFileImputation(options, options.peakgroups_infile, options.do_single_run, options.method)
     else:
         new_exp, multipeptides = runImputeValues(options, options.peakgroups_infile, options.infiles)
-    if options.dry_run: return
+
+    if options.dry_run:
+        return
+
     write_out(new_exp, multipeptides, options.output, options.matrix_outfile, options.do_single_run)
 
 if __name__=="__main__":
