@@ -48,7 +48,6 @@ from ChromatogramTransition import ChromatogramTransition
 from SwathRun import SwathRun
 from SwathRunCollection import SwathRunCollection
 
-DRAW_TRANSITIONS = False
 REALIGN_RUNS = True
 FDR_CUTOFF = 0.01
 ONLY_SHOW_QUANTIFIED = False
@@ -59,6 +58,9 @@ class PrecursorModel():
 
     It is initialized with an ID and knows how to parse the sequence and the
     charge from that string.
+
+    Attributes:
+        runs(str): The chromatogram native id used to infer charge and peptide sequence
     """
 
     def __init__(self, chrom_id):
@@ -66,46 +68,39 @@ class PrecursorModel():
 
     def getCharge(self):
         try:
-            return self.chrom_id.split("/")[1].split("_")[0]
+            return int(self.chrom_id.split("/")[1].split("_")[0])
         except Exception:
             return "NA"
 
     def getFullSequence(self):
         try:
-            return self.chrom_id.split("/")[0].split("_")[-1]
+            assert self.chrom_id.find("/") != -1
+            seq = self.chrom_id.split("/")[0].split("_")[-1]
+            return seq
         except Exception:
-            return "NA"
+            return self.chrom_id
 
 class DataModel(object):
     """The main data model
 
-    It stores the SwathRun objects and can be initialized from a list of files. 
+    It stores the references to individual :class:`.SwathRun` objects and can
+    be initialized from a list of files. 
 
     Attributes:
-        runs:     A list of SwathRun objects
-
-    Methods:
-        get_precursors_tree:  Returns a list of ChromatogramTransition root elements (rows) to display in the left side tree
-                              view. Each element may contain nested ChromatogramTransition elements (tree elements).
-
-        get_runs:             Returns the list of SwathRun objects of this
-                              current data model
-
-        loadFiles:            Load a set of chromatogram files (no peakgroup information).
-
-        loadMixedFiles:       Load a set of chromatogram files (flat file list) and aligned peakgroup files. 
-
-        load_from_yaml:       Load a yaml file containing a mapping of chromatogram files and aligned peakgroup files.
-
+        runs(list of :class:`.SwathRun`): The MS runs which are handled by this class
     """
 
     def __init__(self):
         self.runs = []
+        self.draw_transitions_ = False
 
     #
     ## Getters
     #
     def getStatus(self):
+        """
+        Returns its own status (number of transitions etc.) for the status bar.
+        """
         if len(self.get_runs()) == 0:
             return "Ready"
 
@@ -113,49 +108,105 @@ class DataModel(object):
         for r in self.get_runs():
             tr_cnt += r.getTransitionCount()
 
+        print "Transitions", self.get_runs()[0].getTransitionCount()
+        print "Precursor", len(self.get_runs()[0]._precursor_run_map)
+        print "Peptides", len(self.get_runs()[0]._sequences_mapping)
+        print "Proteins", len(self.get_runs()[0]._protein_mapping)
+        print "total ", tr_cnt 
         return '%s Transitions, %s Peptides (total %s Transitions)' % ( 
             self.get_runs()[0].getTransitionCount(), len(self.get_runs()[0]._sequences_mapping), tr_cnt )
 
     def get_precursor_tree(self):
+        """
+        Returns the data models precursor tree structure
+
+        Returns a list of :class:`.ChromatogramTransition` root elements (rows)
+        to display in the left side tree view. Each element may contain nested
+        :class:`.ChromatogramTransition` elements (tree elements).
+
+        """
         return self._build_tree()
 
     def get_runs(self):
+        """
+        Returns the list of :class:`.SwathRun` objects of this current data model
+
+        Returns
+        -----------
+        list of :class:`.SwathRun`
+            The main content of the class is returned, its list of :class:`.SwathRun`
+        """
         return self.runs
 
+    def getDrawTransitions(self):
+        return self.draw_transitions_
+
+    def setDrawTransitions(self, draw_transitions):
+        self.draw_transitions_ = draw_transitions
+
     #
     ## Data loading
     #
     def loadFiles(self, filenames):
+        """
+        Load a set of chromatogram files (no peakgroup information).
+
+        Args:
+            filenames(list of str): List of filepaths containing the chromatograms
+        """
 
         swathfiles = SwathRunCollection()
         swathfiles.initialize_from_files(filenames)
         self.runs = [run for run in swathfiles.getSwathFiles()]
 
-    def loadMixedFiles(self, rawdata_files, aligned_pg_files):
+    def loadMixedFiles(self, rawdata_files, aligned_pg_files, fileType):
         """ Load files that contain raw data files and aligned peakgroup files.
 
         Since no mapping is present here, we need to infer it from the data.
         Basically, we try to map the column align_runid to the filenames of the
         input .chrom.mzML hoping that the user did not change the filenames.
+
+        Parameters
+        ----------
+        rawdata_files : list of str
+            List of paths to chrom.mzML files
+        aligned_pg_files : list of str
+            List of paths to output files of the FeatureAligner
+        fileType : str
+            Description of the type of file the metadata file (valid: simple, traml, openswath)
         """
 
         print "Input contained no mapping of run_id to the chromatograms."
-        print "Try to infer mapping - if this fails, please provide a yaml input."
+        print "Try to infer mapping for filetype %s - if this fails, please provide a yaml input." % fileType
 
         precursors_mapping = {}
         sequences_mapping = {}
+        protein_mapping = {}
         mapping = {}
-        inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping, sequences_mapping)
+        inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping, sequences_mapping, protein_mapping, fileType=fileType)
         print "Found the following mapping: mapping", mapping
 
         # Read the chromatograms
         swathfiles = SwathRunCollection()
-        swathfiles.initialize_from_chromatograms(mapping, precursors_mapping, sequences_mapping)
+        if ONLY_SHOW_QUANTIFIED:
+            swathfiles.initialize_from_chromatograms(mapping, precursors_mapping, sequences_mapping, protein_mapping)
+        else:
+            swathfiles.initialize_from_chromatograms(mapping)
         self.runs = [run for run in swathfiles.getSwathFiles()]
-        self._read_peakgroup_files(aligned_pg_files, swathfiles)
+        if not fileType in ["simple", "traml"]:
+            self._read_peakgroup_files(aligned_pg_files, swathfiles)
+
         print "Find in total a collection of %s runs." % len(swathfiles.getRunIds() )
                     
     def load_from_yaml(self, yamlfile):
+        """
+        Load a yaml file containing a mapping of chromatogram files and aligned peakgroup files.
+
+        Parameters
+        ----------
+        yamlfile : str
+            Filepath to the yaml file for loading
+        """
 
         import yaml
         data = yaml.load(open(yamlfile) )["AlignedSwathRuns"]
@@ -196,8 +247,9 @@ class DataModel(object):
             if ONLY_SHOW_QUANTIFIED:
                 intersection = set(swathrun.get_all_precursor_ids()).intersection( peakgroup_map.keys() )
                 todelete = set(swathrun.get_all_precursor_ids()).difference(intersection)
-                if len(intersection):
+                if len(intersection) == 0:
                     print "Could not find any intersection between identifiers in your transition file and the provided chromatograms"
+                    print len(intersection)
                 swathrun.remove_precursors(todelete)
 
             # for each precursor in this run, identify the best peakgroup and store the value
@@ -206,11 +258,12 @@ class DataModel(object):
                     continue
 
                 m = peakgroup_map[ precursor_id ]
-                if m.has_peptide(swathrun.runid):
-                    pg = m.get_peptide(swathrun.runid).get_best_peakgroup()
-                    swathrun._range_mapping[precursor_id]       = [ float(pg.get_value("leftWidth")), float(pg.get_value("rightWidth")) ]
-                    swathrun._score_mapping[precursor_id]       = float(pg.get_value("m_score"))
-                    swathrun._intensity_mapping[precursor_id]   = float(pg.get_value("Intensity"))
+                if m.hasPrecursorGroup(swathrun.runid):
+                    for pg in m.getPrecursorGroup(swathrun.runid).getAllPeakgroups():
+                        l,r       = [ float(pg.get_value("leftWidth")), float(pg.get_value("rightWidth")) ]
+                        fdrscore  = float(pg.get_value("m_score"))
+                        intensity = float(pg.get_value("Intensity"))
+                        swathrun.add_peakgroup_data(precursor_id,l,r, fdrscore, intensity)
 
     def _loadFiles_with_peakgroups(self, RawData, aligned_pg_files):
 
@@ -236,14 +289,45 @@ class DataModel(object):
         transformation_collection_.initialize_from_data(reverse=True)
 
     def _build_tree(self):
+        """
+        Build tree of :class:`.ChromatogramTransition` objects for display (see get_precursor_tree)
+        """
 
         peptide_sequences = set([])
         for r in self.get_runs():
             peptide_sequences.update( r.get_all_peptide_sequences() )
 
-        ## The peptide sequences are our top-level items
-        # print "pepseqs", peptide_sequences
+        proteins = set([])
+        for r in self.get_runs():
+            proteins.update( r.get_all_proteins() )
+
+        if len(proteins) == 0:
+            elements = self._build_tree_sequences(peptide_sequences)
+        else:
+
+            elements = []
+            for protein in proteins:
+
+                # get all sequences from all runs
+                sequences = set([])
+                for r in self.get_runs():
+                    sequences.update( r.get_sequence_for_protein(protein) )
+
+                pelements = self._build_tree_sequences(sequences)
+
+                elements.append(ChromatogramTransition(protein,
+                                                       "NA",
+                                                       pelements, 
+                                                       datatype="Protein") )
+
+        return elements
+
+    def _build_tree_sequences(self, peptide_sequences):
+        """
+        Build tree of :class:`.ChromatogramTransition` objects for display (see get_precursor_tree)
+        """
         elements = []
+        pm = PrecursorModel("NA")
         for seq in peptide_sequences:
 
             # get all precursors from all runs
@@ -262,11 +346,27 @@ class DataModel(object):
                 tr_elements = []
                 pm = PrecursorModel(p)
                 for tr in transitions:
-                    if DRAW_TRANSITIONS:
-                        tr_elements.append(ChromatogramTransition(tr, -1, [], fullName=tr, peptideSequence = pm.getFullSequence(), datatype="Transition") )
-                pelements.append(ChromatogramTransition(p, pm.getCharge(), tr_elements, 
-                       peptideSequence = pm.getFullSequence(), datatype="Precursor") )
-            elements.append(ChromatogramTransition(seq, "NA", pelements, datatype="Peptide", 
-                       peptideSequence=pm.getFullSequence()) )
+
+                    # Only add transition data if individual transitions should be drawn
+                    if self.draw_transitions_:
+                        tr_elements.append(
+                            ChromatogramTransition(tr,
+                                                   -1,
+                                                   [],
+                                                   fullName=tr,
+                                                   peptideSequence = pm.getFullSequence(),
+                                                   datatype="Transition") )
+
+                pelements.append(ChromatogramTransition(p,
+                                                        pm.getCharge(),
+                                                        tr_elements, 
+                                                        peptideSequence = pm.getFullSequence(),
+                                                        datatype="Precursor") )
+
+            elements.append(ChromatogramTransition(seq,
+                                                   "NA",
+                                                   pelements, 
+                                                   datatype="Peptide",
+                                                   peptideSequence=pm.getFullSequence()) )
         return elements
 
