@@ -60,7 +60,6 @@ def getDistanceMatrix(exp, multipeptides, spl_aligner, singleRowId=None):
             given that there are enough points.
 
     Returns:
-        None
         numpy (n x n) matrix(float): distance matrix
     """
 
@@ -107,25 +106,25 @@ class TreeConsensusAlignment():
 
     For example, consider a case of 5 LC-MS/MS runs where 6 different feature
     (peakgroups) were found in each run (not all peakgroups were found in all
-    runs):
+    runs)::
 
-    Run 1:
-    ---------- pg1_1 ------- pg1_2 --- pg1_3 - pg1_4 --------- pg1_5 ------- pg1_6
+        Run 1:
+        ---------- pg1_1 ------- pg1_2 --- pg1_3 - pg1_4 --------- pg1_5 ------- pg1_6
 
-    Run 2:
-    ----------- pg2_1 ------- pg2_2 --- pg2_3 - pg2_4 --------- pg2_5 ------ pg2_6
+        Run 2:
+        ----------- pg2_1 ------- pg2_2 --- pg2_3 - pg2_4 --------- pg2_5 ------ pg2_6
 
-    Run 3:
-    ----- pg3_0 ---------- pg3_1 ---- pg3_2 - pg3_3 - pg3_4 ----- pg3_5 ---- pg3_6
+        Run 3:
+        ----- pg3_0 ---------- pg3_1 ---- pg3_2 - pg3_3 - pg3_4 ----- pg3_5 ---- pg3_6
 
-    Run 4:
-    ----------- pg4_0 ---------- pg4_1 ------- pg4_2 --- pg4_3 - pg4_4 --------- pg4_5
+        Run 4:
+        ----------- pg4_0 ---------- pg4_1 ------- pg4_2 --- pg4_3 - pg4_4 --------- pg4_5
 
-    Run 5:
-    -- pg5_0 ---------- pg5_1 ------- pg5_2 --- pg5_3 - pg5_4 --------- pg5_5 
+        Run 5:
+        -- pg5_0 ---------- pg5_1 ------- pg5_2 --- pg5_3 - pg5_4 --------- pg5_5 
 
 
-    Assume that the corresponding MST looks like this:
+    Assume that the corresponding MST looks like this::
 
                                /-- Run4
         Run1 -- Run2 -- Run3 --
@@ -134,19 +133,20 @@ class TreeConsensusAlignment():
     This is a case where Run1 and Run2 are very similar and Run3 and Run4 are
     rather similar and should be easy to align. The algorithm will start with
     the "best" peakgroup overall (having the best probability score), assume
-    this peakgroups is pg1_1 from Run 1. The algorithm will then use the
-    alignment Run1-Run2 to infer that pg2_1 is the same signal as pg1_1 and add
-    it to the group. Specifically, it will select the highest-scoring peakgroup
-    within a narrow RT-window (max_rt_diff) in Run2 - note that if the
-    RT-window is too wide, there is a certain chance of mis-matching, e.g. pg_2
-    will be selected instead of pg2_1.  The alignment Run2-Run3 will be used to
-    add pg3_1. Then a bifurcation in the tree occurs and Run3-Run4 as well as
-    Run3-Run5 will be used to infer the identity of pg4_1 and pg5_1 and add
-    them to the cluster.  In the end, the algorithm will report (pg1_1, pg2_1,
-    pg3_1, pg4_1, pg5_1) as a consistent cluster across multiple runs. This
-    process can be repeated with the next best peakgroup that is not yet part
-    of a cluster (e.g. pg1_2) until no more peakgroups are left (no more
-    peakgroups having a score below fdr_cutoff).
+    this peakgroups is ``pg1_1`` from Run 1. The algorithm will then use the
+    alignment Run1-Run2 to infer that ``pg2_1`` is the same signal as ``pg1_1``
+    and add it to the group. Specifically, it will select the highest-scoring
+    peakgroup within a narrow RT-window (max_rt_diff) in Run2 - note that if
+    the RT-window is too wide, there is a certain chance of mis-matching, e.g.
+    ``pg_2`` will be selected instead of ``pg2_1``.  The alignment Run2-Run3
+    will be used to add ``pg3_1``. Then a bifurcation in the tree occurs and
+    Run3-Run4 as well as Run3-Run5 will be used to infer the identity of
+    ``pg4_1`` and ``pg5_1`` and add them to the cluster.  In the end, the
+    algorithm will report (pg1_1, pg2_1, ``pg3_1``, ``pg4_1``, ``pg5_1``) as a
+    consistent cluster across multiple runs. This process can be repeated with
+    the next best peakgroup that is not yet part of a cluster (e.g. ``pg1_2``)
+    until no more peakgroups are left (no more peakgroups having a score below
+    fdr_cutoff).
 
     Note how the algorithm only used binary alignments and purely local
     alignments of the runs that are most close to each other. This stands in
@@ -156,7 +156,7 @@ class TreeConsensusAlignment():
     and could lead to whole subtrees that are wrongly aligned.
     """
 
-    def __init__(self, max_rt_diff, fdr_cutoff, aligned_fdr_cutoff,
+    def __init__(self, max_rt_diff, fdr_cutoff, aligned_fdr_cutoff, rt_diff_isotope=-1,
                  correctRT_using_pg=False, stdev_max_rt_per_run=None,
                  use_local_stdev=False, verbose=False):
         """ Initialization with parameters
@@ -184,6 +184,7 @@ class TreeConsensusAlignment():
         self._stdev_max_rt_per_run = stdev_max_rt_per_run
         self._use_local_stdev = use_local_stdev
         self.verbose = verbose
+        self.max_rt_diff_isotope = rt_diff_isotope
 
         # Number of multiple possible alignments calls (more than one possibility)
         self.nr_multiple_align = 0
@@ -198,11 +199,11 @@ class TreeConsensusAlignment():
         the first cluster will be reported).
 
         Args:
-            multipeptides(list(Multipeptide)): a list of
+            multipeptides(list of :class:`.Multipeptide`): a list of
                 multipeptides on which the alignment should be performed. After
                 alignment, each peakgroup that should be quantified can be
                 retrieved by calling get_selected_peakgroups() on the multipeptide.
-            tree(list(tuple)): a minimum spanning tree (MST) represented as
+            tree(list of tuple): a minimum spanning tree (MST) represented as
                 list of edges (for example [('0', '1'), ('1', '2')] ). Node names
                 need to correspond to run ids.
             tr_data(format.TransformationCollection.LightTransformationData):
@@ -244,9 +245,8 @@ class TreeConsensusAlignment():
             tree(list(tuple)): a minimum spanning tree (MST) represented as
                 list of edges (for example [('0', '1'), ('1', '2')] ). Node names
                 need to correspond to run ids.
-            tr_data(format.TransformationCollection.LightTransformationData):
-                structure to hold binary transformations between two different
-                retention time spaces
+            tr_data(:class:`.LightTransformationData`): structure to hold
+                binary transformations between two different retention time spaces
 
         Returns:
             None
@@ -254,25 +254,26 @@ class TreeConsensusAlignment():
 
         from msproteomicstoolslib.algorithms.alignment.AlignmentAlgorithm import Cluster
         for m in multipeptides:
+
             if self.verbose: 
-                print "00000000000000000000000000000000000 new peptide (cluster)", m.get_peptides()[0].get_id()
+                print "00000000000000000000000000000000000 new peptide (cluster)", m.getPrecursorGroups()[0].getPeptideGroupLabel()
 
             last_cluster = []
             already_seen = set([])
-            stillLeft = [b for a in m.get_peptides() for b in a.get_all_peakgroups() 
+            stillLeft = [b for a in m.getPrecursorGroups() for b in a.getAllPeakgroups() 
                          if b.get_feature_id() + b.peptide.get_id() not in already_seen 
                          and b.get_fdr_score() < self._fdr_cutoff]
             clusters = []
             while len(stillLeft) > 0:
                 best = min(stillLeft, key=lambda x: float(x.get_fdr_score()))
                 if self.verbose: 
-                    print "111111111111111111111111 new cluster"
-                    print " Best", best.print_out(), "from run", best.peptide.run.get_id()
+                    print " == alignAllCluster: new cluster initialized"
+                    print " == Best", best.print_out(), "from run", best.peptide.run.get_id()
 
                 last_cluster = self._findAllPGForSeed(tree, tr_data, m, best, already_seen)
                 already_seen.update( set([ b.get_feature_id() + b.peptide.get_id() 
                                           for b in last_cluster if b is not None]) )
-                stillLeft = [b for a in m.get_peptides() for b in a.get_all_peakgroups() 
+                stillLeft = [b for a in m.getPrecursorGroups() for b in a.getAllPeakgroups() 
                              if b.get_feature_id() + b.peptide.get_id() not in already_seen 
                              and b.get_fdr_score() < self._fdr_cutoff]
                 clusters.append(Cluster(last_cluster))
@@ -310,9 +311,8 @@ class TreeConsensusAlignment():
             tree(list(tuple)): a minimum spanning tree (MST) represented as
                 list of edges (for example [('0', '1'), ('1', '2')] ). Node names
                 need to correspond to run ids.
-            tr_data(format.TransformationCollection.LightTransformationData):
-                structure to hold binary transformations between two different
-                retention time spaces
+            tr_data(:class:`.LightTransformationData`): structure to hold
+                binary transformations between two different retention time spaces
             m(Multipeptide): one multipeptides on which the alignment should be performed
             seed(PeakGroupBase): one peakgroup chosen as the seed
             already_seen(dict): list of peakgroups already aligned (e.g. in a
@@ -322,8 +322,8 @@ class TreeConsensusAlignment():
             list(PeakGroupBase): List of peakgroups belonging to this cluster
         """
         if self.verbose: 
-            print "111111111111111111111111 new cluster"
-            print " Seed", seed.print_out(), "from run", seed.peptide.run.get_id()
+            print "111111111111111111111111 findAllPGForSeed started"
+            print "  Seed", seed.print_out(), "from run", seed.peptide.run.get_id()
 
         seed_rt = seed.get_normalized_retentiontime()
 
@@ -336,19 +336,45 @@ class TreeConsensusAlignment():
             for e1, e2 in tree:
                 if e1 in visited.keys() and not e2 in visited.keys():
                     if self.verbose:
-                        print "try to align", e2, "from", e1
+                        print "  try to align", e2, "from already known node", e1
                     newPG, rt = self._findBestPG(m, e1, e2, tr_data, rt_map[e1], already_seen)
                     rt_map[e2] = rt
                     visited[e2] = newPG
                 if e2 in visited.keys() and not e1 in visited.keys():
                     if self.verbose: 
-                        print "try to align", e1, "from", e2
+                        print "  try to align", e1, "from", e2
                     newPG, rt = self._findBestPG(m, e2, e1, tr_data, rt_map[e2], already_seen)
                     rt_map[e1] = rt
                     visited[e1] = newPG
-        return [pg for pg in visited.values() if pg is not None]
 
-    def _findBestPG(self, m,  source, target, tr_data, source_rt, already_seen):
+        # Now in each run at most one (zero or one) peakgroup got selected for
+        # the current peptide label group. This means that for each run, either
+        # heavy or light was selected (but not both) and now we should align
+        # the other isotopic channels as well.
+        if self.verbose: 
+            print "Re-align isotopic channels:"
+
+        isotopically_added_pg = []
+        for pg in visited.values():
+            if pg is not None:
+
+                # Iterate through all sibling peptides (same peptide but
+                # different isotopic composition) and pick a peak using the
+                # already aligned channel as a reference.
+                ref_peptide = pg.peptide
+                for pep in ref_peptide.precursor_group:
+                    if ref_peptide != pep:
+                        if self.verbose: 
+                            print "  Using reference %s at RT %s to align peptide %s." % (ref_peptide, pg.get_normalized_retentiontime(), pep)
+                        newPG, rt = self._findBestPGFromTemplate(pg.get_normalized_retentiontime(), pep, self.max_rt_diff_isotope, already_seen)
+                        isotopically_added_pg.append(newPG)
+
+        if self.verbose: 
+            print "Done with re-alignment of isotopic channels"
+
+        return [pg for pg in visited.values() + isotopically_added_pg if pg is not None]
+
+    def _findBestPG(self, m, source, target, tr_data, source_rt, already_seen):
         """Find (best) matching peakgroup in "target" which matches to the source_rt RT.
 
         Args:
@@ -368,12 +394,12 @@ class TreeConsensusAlignment():
         # Get expected RT (transformation of source into target domain)
         expected_rt = tr_data.getTrafo(source, target).predict([source_rt])[0]
         if self.verbose:
-            print "Expected RT", expected_rt, " (source RT)", source_rt
-            print " --- and back again :::  ", tr_data.getTrafo(target, source).predict([expected_rt])[0]
+            print "  Expected RT", expected_rt, " (source RT)", source_rt
+            print "  --- and back again :::  ", tr_data.getTrafo(target, source).predict([expected_rt])[0]
 
         # If there is no peptide present in the target run, we simply return
         # the expected retention time.
-        if not m.has_peptide(target):
+        if not m.hasPrecursorGroup(target):
             return None, expected_rt
 
         max_rt_diff = self._max_rt_diff
@@ -388,11 +414,16 @@ class TreeConsensusAlignment():
             max_rt_diff = max(self._max_rt_diff, max_rt_diff)
 
         if self.verbose:
-            print "Used rt diff:", max_rt_diff
+            print "  Used rt diff:", max_rt_diff
 
+        return self. _findBestPGFromTemplate(expected_rt, m.getPrecursorGroup(target), max_rt_diff, already_seen)
+
+    def _findBestPGFromTemplate(self, expected_rt, target_peptide, max_rt_diff, already_seen):
+        """Find (best) matching peakgroup in "target" which matches to the source_rt RT.
+
+        """
         # Select matching peakgroups from the target run (within the user-defined maximal rt deviation)
-        target_p = m.get_peptide(target)
-        matching_peakgroups = [pg_ for pg_ in target_p.get_all_peakgroups() 
+        matching_peakgroups = [pg_ for pg_ in target_peptide.getAllPeakgroups() 
             if (abs(float(pg_.get_normalized_retentiontime()) - float(expected_rt)) < max_rt_diff) and
                 pg_.get_fdr_score() < self._aligned_fdr_cutoff and 
                 pg_.get_feature_id() + pg_.peptide.get_id() not in already_seen]
@@ -408,8 +439,8 @@ class TreeConsensusAlignment():
         # Printing for debug mode
         if self.verbose:
             closestPG = min(matching_peakgroups, key=lambda x: abs(float(x.get_normalized_retentiontime()) - expected_rt))
-            print "  closest:", closestPG.print_out(), "diff", abs(closestPG.get_normalized_retentiontime() - expected_rt)
-            print "  bestScoring:", bestScoringPG.print_out(), "diff", abs(bestScoringPG.get_normalized_retentiontime() - expected_rt)
+            print "    closest:", closestPG.print_out(), "diff", abs(closestPG.get_normalized_retentiontime() - expected_rt)
+            print "    bestScoring:", bestScoringPG.print_out(), "diff", abs(bestScoringPG.get_normalized_retentiontime() - expected_rt)
             print
 
         if len([pg_ for pg_ in matching_peakgroups if pg_.get_fdr_score() < self._aligned_fdr_cutoff]) > 1:
