@@ -44,10 +44,15 @@ from msproteomicstoolslib.format.SWATHScoringReader import Run
 
 class MRExperiment(object):
     """
-    An MR (multirun) Experiment is a container for multiple experimental runs -
-    some of which may contain the same precursors.
+    An MR (multirun) Experiment is a container for multiple experimental runs.
 
-    # Read the files
+    In some of the runs the same peptidde precursors may be identified and the
+    job of this object is to keep track of these experiments and the identified
+    precursors across multiple runs.
+
+    Example usage:
+
+    >>> # Read the files
     >>> fdr_cutoff = 0.01 # 1% FDR
     >>> reader = SWATHScoringReader.newReader(infiles, "openswath")
     >>> this_exp = Experiment()
@@ -64,7 +69,7 @@ class MRExperiment(object):
         """Initialize with a set of runs.
 
         Args:
-            runs(list(SWATHScoringReader.Run))
+            runs(list of :class:`.Run`) : A set of runs
         """
         self.runs = runs
 
@@ -74,6 +79,17 @@ class MRExperiment(object):
         Find all precursors that are above the fdr cutoff in each run and build
         a union of those precursors. Then search for each of those precursors
         in all the other runs and build a multipeptide / multiprecursor.
+
+        Parameters
+        ----------
+        fdr_cutoff : float
+            A cutoff in fdr (between 0 and 1) to use for the alignment. Each
+            generated Multipeptide needs to have at least one member who is below
+            the cutoff.
+        verbose : bool
+            Whether to be verbose or not
+        verbosity : int
+            How verbose to be
         """
         union_transition_groups = []
         union_proteins = []
@@ -82,8 +98,8 @@ class MRExperiment(object):
             if verbose or verbosity >= 10: 
                 stdout.write("\rParsing run %s out of %s" % (i+1, len(self.runs) ))
                 stdout.flush()
-            union_target_transition_groups.append( [peak.peptide.get_id() for peak in r.get_best_peaks_with_cutoff(fdr_cutoff) if not peak.peptide.get_decoy()] )
-            union_transition_groups.append( [peak.peptide.get_id() for peak in r.get_best_peaks_with_cutoff(fdr_cutoff)] )
+            union_target_transition_groups.append( [peak.peptide.precursor_group.getPeptideGroupLabel() for peak in r.get_best_peaks_with_cutoff(fdr_cutoff) if not peak.peptide.get_decoy()] )
+            union_transition_groups.append( [peak.peptide.precursor_group.getPeptideGroupLabel() for peak in r.get_best_peaks_with_cutoff(fdr_cutoff)] )
             union_proteins.append( list(set([peak.peptide.protein_name for peak in r.get_best_peaks_with_cutoff(fdr_cutoff) if not peak.peptide.get_decoy()])) )
         if verbose or verbosity >= 10: stdout.write("\r\r\n") # clean up
 
@@ -119,8 +135,8 @@ class MRExperiment(object):
         for peptide_id in self.union_transition_groups_set:
             m = Multipeptide()
             for r in self.runs:
-                peptide = r.get_peptide(peptide_id)
-                m.insert(r.get_id(), peptide)
+                precursor_group = r.getPrecursorGroup(peptide_id)
+                m.insert(r.get_id(), precursor_group)
             m.set_nr_runs(len(self.runs))
             multipeptides.append(m)
         return multipeptides
