@@ -80,14 +80,24 @@ class SWATHScoringReader:
         raise Exception("Abstract method")
 
     @staticmethod
-    def newReader(infiles, filetype, readmethod="minimal", readfilter=ReadFilter(), errorHandling="strict"):
-        """Factory to create a new reader"""
+    def newReader(infiles, filetype, readmethod="minimal",
+                  readfilter=ReadFilter(), errorHandling="strict", enable_isotopic_grouping=False):
+        """
+        newReader(infiles, filetype, readmethod="minimal", readfilter=ReadFilter(), errorHandling="strict", enable_isotopic_grouping=False)
+
+        Factory to create a new reader
+        
+        """
         if filetype  == "openswath": 
-            return OpenSWATH_SWATHScoringReader(infiles, readmethod, readfilter, errorHandling)
+            return OpenSWATH_SWATHScoringReader(infiles, readmethod,
+                                                readfilter, errorHandling,
+                                                enable_isotopic_grouping=enable_isotopic_grouping)
         elif filetype  == "mprophet": 
             return mProphet_SWATHScoringReader(infiles, readmethod, readfilter)
         elif filetype  == "peakview": 
             return Peakview_SWATHScoringReader(infiles, readmethod, readfilter)
+        elif filetype  == "peakview_preprocess": 
+            return PeakviewPP_SWATHScoringReader(infiles, readmethod, readfilter)
         else:
             raise Exception("Unknown filetype '%s', allowed types are %s" % (decoy, str(filetypes) ) )
 
@@ -172,8 +182,11 @@ class SWATHScoringReader:
       return runs
 
 class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
+    """
+    Parser for OpenSWATH output
+    """
 
-    def __init__(self, infiles, readmethod="minimal", readfilter=ReadFilter(), errorHandling="strict"):
+    def __init__(self, infiles, readmethod="minimal", readfilter=ReadFilter(), errorHandling="strict", enable_isotopic_grouping=False):
         self.infiles = infiles
         self.run_id_name = "run_id"
         self.readmethod = readmethod
@@ -191,6 +204,11 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
             self.Precursor = GeneralPrecursor
             self.PeakGroup = GeneralPeakGroup
 
+        if enable_isotopic_grouping:
+            self.peptide_group_label_name = "peptide_group_label"
+        else:
+            self.peptide_group_label_name = "transition_group_id"
+
     def parse_row(self, run, this_row, read_exp_RT):
         decoy_name = "decoy"
         fdr_score_name = "m_score"
@@ -219,6 +237,10 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
         trgr_id = this_row[run.header_dict[unique_peakgroup_id_name]]
         unique_peakgroup_id = this_row[run.header_dict[unique_peakgroup_id_name]]
         sequence = this_row[run.header_dict[self.sequence_col]]
+        peptide_group_label = trgr_id
+
+        if self.peptide_group_label_name in run.header_dict: 
+            peptide_group_label = this_row[run.header_dict[self.peptide_group_label_name]]
 
         # Attributes that only need to be present in strict mode
         diff_from_assay_seconds = -1
@@ -227,10 +249,12 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
         thisid = -1
         try:
             fdr_score = float(this_row[run.header_dict[fdr_score_name]])
+            #fdr_score = 0.0001
             protein_name = this_row[run.header_dict[protein_id_col]]
             thisid = this_row[run.header_dict[unique_feature_id_name]]
             diff_from_assay_seconds = float(this_row[run.header_dict[diff_from_assay_in_sec_name]])
             d_score = float(this_row[run.header_dict[dscore_name]])
+            #d_score = 2
         except KeyError:
             if self.errorHandling == "strict": 
                 raise Exception("Did not find essential column.")
@@ -242,35 +266,39 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
         if "decoy" in run.header_dict:
             decoy = this_row[run.header_dict[decoy_name]]
 
-        # If the peptide does not yet exist
-        if not run.all_peptides.has_key(trgr_id):
+        # If the peptide does not yet exist, generate it
+        if not run.hasPrecursor(peptide_group_label, trgr_id):
           p = self.Precursor(trgr_id, run)
           p.protein_name = protein_name
           p.sequence = sequence
           p.set_decoy(decoy)
-          run.all_peptides[trgr_id] = p
+          run.addPrecursor(p, peptide_group_label)
 
         if self.readmethod == "minimal":
           peakgroup_tuple = (thisid, fdr_score, diff_from_assay_seconds, intensity, d_score)
-          run.all_peptides[trgr_id].add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id, cluster_id)
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id, cluster_id)
         elif self.readmethod == "gui":
           leftWidth = this_row[run.header_dict[left_width_name]]
           rightWidth = this_row[run.header_dict[right_width_name]]
           charge = this_row[run.header_dict[charge_name]]
-          peakgroup = self.PeakGroup(fdr_score, intensity, leftWidth, rightWidth, run.all_peptides[trgr_id])
-          run.all_peptides[trgr_id].add_peakgroup(peakgroup)
+          peakgroup = self.PeakGroup(fdr_score, intensity, leftWidth, rightWidth, run.getPrecursor(peptide_group_label, trgr_id))
+          peakgroup.charge = charge
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup(peakgroup)
         elif self.readmethod == "complete":
-          peakgroup = self.PeakGroup(this_row, run, run.all_peptides[trgr_id])
+          peakgroup = self.PeakGroup(this_row, run, run.getPrecursor(peptide_group_label, trgr_id))
           peakgroup.set_normalized_retentiontime(diff_from_assay_seconds)
           peakgroup.set_fdr_score(fdr_score)
           peakgroup.set_feature_id(thisid)
           peakgroup.set_intensity(intensity)
           peakgroup.setClusterID(cluster_id)
-          run.all_peptides[trgr_id].add_peakgroup(peakgroup)
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup(peakgroup)
 
 class mProphet_SWATHScoringReader(SWATHScoringReader):
+    """
+    Parser for mProphet output
+    """
 
-    def __init__(self, infiles, readmethod="minimal", readfilter=ReadFilter()):
+    def __init__(self, infiles, readmethod="minimal", readfilter=ReadFilter(), enable_isotopic_grouping=False):
         self.infiles = infiles
         self.run_id_name = "run_id"
         self.readmethod = readmethod
@@ -282,6 +310,9 @@ class mProphet_SWATHScoringReader(SWATHScoringReader):
             self.Precursor = GeneralPrecursor
             self.PeakGroup = GeneralPeakGroup
 
+        if enable_isotopic_grouping:
+            raise Exception("Cannot use isotopic grouping with mProphet data.")
+
     def parse_row(self, run, this_row, read_exp_RT):
         decoy_name = "decoy"
         fdr_score_name = "m_score"
@@ -320,26 +351,31 @@ class mProphet_SWATHScoringReader(SWATHScoringReader):
             decoy = this_row[run.header_dict[decoy_name]]
         run_id = this_row[run.header_dict[run_id_name]]
 
-        if not run.all_peptides.has_key(trgr_id):
+        # If the peptide does not yet exist, generate it
+        peptide_group_label = trgr_id
+        if not run.hasPrecursor(peptide_group_label, trgr_id):
           p = self.Precursor(trgr_id, run)
           p.protein_name = protein_name
           p.sequence = sequence
           p.set_decoy(decoy)
-          run.all_peptides[trgr_id] = p
+          run.addPrecursor(p, peptide_group_label)
         if self.readmethod == "minimal":
           peakgroup_tuple = (thisid, fdr_score, diff_from_assay_seconds, intensity)
-          run.all_peptides[trgr_id].add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id)
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id)
         else:
-          peakgroup = self.PeakGroup(this_row, run, run.all_peptides[trgr_id])
+          peakgroup = self.PeakGroup(this_row, run, run.getPrecursor(peptide_group_label, trgr_id))
           peakgroup.set_normalized_retentiontime(diff_from_assay_seconds)
           peakgroup.set_fdr_score(fdr_score)
           peakgroup.set_feature_id(thisid)
           peakgroup.set_intensity(intensity)
-          run.all_peptides[trgr_id].add_peakgroup(peakgroup)
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup(peakgroup)
 
 class Peakview_SWATHScoringReader(SWATHScoringReader):
+    """
+    Parser for Peakview output
+    """
 
-    def __init__(self, infiles, readmethod="minimal", readfilter=ReadFilter()):
+    def __init__(self, infiles, readmethod="minimal", readfilter=ReadFilter(), enable_isotopic_grouping=False):
         self.infiles = infiles
         self.run_id_name = "Sample"
         self.aligned_run_id_name = "align_runid"
@@ -349,6 +385,12 @@ class Peakview_SWATHScoringReader(SWATHScoringReader):
         # Only minimal reading is implemented
         if readmethod == "minimal":
             self.Precursor = Precursor
+        else:
+            raise Exception("Cannot only use readmethod minimal Peakview data.")
+
+        if enable_isotopic_grouping:
+            raise Exception("Cannot use isotopic grouping with Peakview data.")
+
 
     def parse_row(self, run, this_row, read_exp_RT):
         decoy_name = "Decoy"
@@ -398,24 +440,241 @@ class Peakview_SWATHScoringReader(SWATHScoringReader):
             decoy = this_row[run.header_dict[decoy_name]]
         run_id = this_row[run.header_dict[run_id_name]]
 
-        if not run.all_peptides.has_key(trgr_id):
+        # If the peptide does not yet exist, generate it
+        peptide_group_label = trgr_id
+        if not run.hasPrecursor(peptide_group_label, trgr_id):
           p = self.Precursor(trgr_id, run)
           p.protein_name = protein_name
           p.sequence = sequence
           p.set_decoy(decoy)
-          run.all_peptides[trgr_id] = p
+          run.addPrecursor(p, peptide_group_label)
           if verb: print "add peptide", trgr_id
 
         # Only minimal reading is implemented
         if self.readmethod == "minimal":
           if verb: print "append tuple", peakgroup_tuple
           peakgroup_tuple = (thisid, fdr_score, diff_from_assay_seconds,intensity)
-          run.all_peptides[trgr_id].add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id)
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id)
         else:
             raise NotImplemented
 
+class PeakviewPP_SWATHScoringReader(Peakview_SWATHScoringReader):
+    """
+    Parser for Peakview output
+    """
+
+    def __init__(self, infiles, readmethod="minimal", readfilter=ReadFilter(), enable_isotopic_grouping=False):
+        self.infiles = infiles
+        self.run_id_name = "Sample"
+        self.aligned_run_id_name = "align_runid"
+        self.readmethod = readmethod
+        self.readfilter = readfilter
+        self.errorHandling = "strict"
+
+        # Only minimal reading is implemented
+        if readmethod == "minimal":
+            self.Precursor = Precursor
+        else:
+            raise Exception("Cannot only use readmethod minimal Peakview data.")
+
+        if enable_isotopic_grouping:
+            raise Exception("Cannot use isotopic grouping with Peakview data.")
+        else:
+            self.peptide_group_label_name = "Pep Index"
+
+    def parse_row(self, run, this_row, read_exp_RT):
+        decoy_name = "decoy"
+        fdr_score_name = "m_score"
+        ## dscore_name = "d_score"
+        unique_peakgroup_id_name = "transition_group_id"
+        ## diff_from_assay_in_sec_name = "delta_rt"
+        run_id_name = "run_id"
+        protein_id_col = "ProteinName"
+        intensity_name = "Intensity"
+        decoy = "FALSE"
+        # left_width_name = "leftWidth"
+        # right_width_name = "rightWidth"
+        charge_name = "Charge"
+        cluster_id = -1
+
+        decoy_name = "Decoy"
+        fdr_score_name = "Score" # note this score is better when higher
+        unique_peakgroup_id_name = "Pep Index"
+        diff_from_assay_in_sec_name = "Median RT"
+        run_id_name = "Sample"
+        protein_id_col = "Protein"
+        self.sequence_col = "Peptide"
+        unique_feature_id_name = "preprocess_id"
+        decoy = "FALSE"
+        intensity_name = "MaxPeak Intensity"
+        charge_name = "Precursor Charge"
+        d_score = 2
+
+        # use the aligned retention time if it is available!
+        if "aligned_rt" in run.header_dict: 
+            diff_from_assay_in_sec_name = "aligned_rt" ## use this if it is present
+        # if we want to re-do the re-alignment, we just use the "regular" retention time
+        if read_exp_RT: 
+            diff_from_assay_in_sec_name = "Median RT"
+        if "align_clusterid" in run.header_dict: 
+            cluster_id = int(this_row[run.header_dict["align_clusterid"]])
+
+        trgr_id = this_row[run.header_dict[unique_peakgroup_id_name]]
+        unique_peakgroup_id = this_row[run.header_dict[unique_peakgroup_id_name]]
+        sequence = this_row[run.header_dict[self.sequence_col]]
+        peptide_group_label = trgr_id
+
+        if self.peptide_group_label_name in run.header_dict: 
+            peptide_group_label = this_row[run.header_dict[self.peptide_group_label_name]]
+
+        # Attributes that only need to be present in strict mode
+        diff_from_assay_seconds = -1
+        fdr_score = -1
+        protein_name = "NA"
+        thisid = -1
+        try:
+            fdr_score = float(this_row[run.header_dict[fdr_score_name]])
+            protein_name = this_row[run.header_dict[protein_id_col]]
+            thisid = this_row[run.header_dict[unique_feature_id_name]]
+            diff_from_assay_seconds = float(this_row[run.header_dict[diff_from_assay_in_sec_name]])
+        except KeyError:
+            if self.errorHandling == "strict": 
+                raise Exception("Did not find essential column.")
+
+        # compute 1/score to have a score that gets better when smaller
+        fdr_score = 1/fdr_score
+
+        # Optional attributes
+        intensity = -1
+        if run.header_dict.has_key(intensity_name):
+            intensity = float(this_row[run.header_dict[intensity_name]])
+        if "Decoy" in run.header_dict:
+            decoy = this_row[run.header_dict[decoy_name]]
+            if decoy in ["True", "TRUE"]:
+                decoy = "TRUE"
+            elif decoy in ["False", "FALSE"]:
+                decoy = "FALSE"
+
+        # If the peptide does not yet exist, generate it
+        if not run.hasPrecursor(peptide_group_label, trgr_id):
+          p = self.Precursor(trgr_id, run)
+          p.protein_name = protein_name
+          p.sequence = sequence
+          p.set_decoy(decoy)
+          run.addPrecursor(p, peptide_group_label)
+
+        if self.readmethod == "minimal":
+          peakgroup_tuple = (thisid, fdr_score, diff_from_assay_seconds, intensity, d_score)
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id, cluster_id)
+        elif self.readmethod == "gui":
+          leftWidth = this_row[run.header_dict[left_width_name]]
+          rightWidth = this_row[run.header_dict[right_width_name]]
+          charge = this_row[run.header_dict[charge_name]]
+          peakgroup = self.PeakGroup(fdr_score, intensity, leftWidth, rightWidth, run.getPrecursor(peptide_group_label, trgr_id))
+          peakgroup.charge = charge
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup(peakgroup)
+        elif self.readmethod == "complete":
+          peakgroup = self.PeakGroup(this_row, run, run.getPrecursor(peptide_group_label, trgr_id))
+          peakgroup.set_normalized_retentiontime(diff_from_assay_seconds)
+          peakgroup.set_fdr_score(fdr_score)
+          peakgroup.set_feature_id(thisid)
+          peakgroup.set_intensity(intensity)
+          peakgroup.setClusterID(cluster_id)
+          run.getPrecursor(peptide_group_label, trgr_id).add_peakgroup(peakgroup)
+
+def simpleInferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
+                 sequences_mapping, protein_mapping, verbose=False):
+
+    assert len(aligned_pg_files) == 1, "There should only be one file in simple mode"
+    f = aligned_pg_files[0]
+
+    # Produce simple mapping between runs and files (assume each file is one run)
+    for i,raw in enumerate(rawdata_files):
+        mapping[str(i)] = [ raw ]
+
+    # Get the compression
+    if f.endswith('.gz'):
+        import gzip 
+        filehandler = gzip.open(f,'rb')
+    else:
+        filehandler = open(f)
+
+    # Get the dialect
+    dialect = csv.Sniffer().sniff(filehandler.readline(), [',',';','\t'])
+    filehandler.seek(0) 
+    reader = csv.reader(filehandler, dialect)
+    header = reader.next()
+
+    header_dict = {}
+    for i,n in enumerate(header):
+        header_dict[n] = i
+
+    for this_row in reader:
+        peptide_name = this_row [ header_dict["chromatogram_super_group_id"]]
+        precursor_name = this_row [ header_dict["chromatogram_group_id"]]
+        transition_name = this_row [ header_dict["chromatogram_id"]]
+
+        # Fill the sequence mapping
+        tmp = sequences_mapping.get(peptide_name, [])
+        if precursor_name not in tmp:
+            tmp.append(precursor_name)
+        sequences_mapping[peptide_name] = tmp
+
+        # Fill the precursor mapping
+        tmp = precursors_mapping.get(precursor_name, [])
+        if transition_name not in tmp:
+            tmp.append(transition_name)
+        precursors_mapping[precursor_name] = tmp
+
+def tramlInferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
+                 sequences_mapping, protein_mapping, verbose=False):
+    try:
+        import pyopenms
+    except ImportError as e:
+        print "\nError!"
+        print "Could not import pyOpenMS while trying to load a TraML file - please make sure pyOpenMS is installed."
+        print "pyOpenMS is available from https://pypi.python.org/pypi/pyopenms"
+        print 
+        raise e
+
+    assert len(aligned_pg_files) == 1, "There should only be one file in simple mode"
+    f = aligned_pg_files[0]
+
+    # Produce simple mapping between runs and files (assume each file is one run)
+    for i,raw in enumerate(rawdata_files):
+        mapping[str(i)] = [ raw ]
+
+    targexp = pyopenms.TargetedExperiment()
+    pyopenms.TraMLFile().load(f, targexp)
+
+    for peptide_precursor in targexp.getPeptides():
+
+        # Fill the protein mapping
+        protein_id = peptide_precursor.protein_refs
+        if len(protein_id) > 0:
+            protein_id = protein_id[0] # take the first one ... 
+
+        tmp = protein_mapping.get(protein_id, [])
+        if peptide_precursor.sequence not in tmp:
+            tmp.append(peptide_precursor.sequence)
+        protein_mapping[protein_id] = tmp
+
+        # Fill the sequence mapping
+        tmp = sequences_mapping.get(peptide_precursor.sequence, [])
+        if peptide_precursor.id not in tmp:
+            tmp.append(peptide_precursor.id)
+        sequences_mapping[peptide_precursor.sequence] = tmp
+
+    for transition in targexp.getTransitions():
+
+        # Fill the precursor mapping
+        tmp = precursors_mapping.get(transition.getPeptideRef(), [])
+        if transition.getPeptideRef() not in tmp:
+            tmp.append(transition.getNativeID())
+        precursors_mapping[transition.getPeptideRef()] = tmp
+
 def inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
-                 sequences_mapping, verbose=False, throwOnMismatch=False):
+                 sequences_mapping, protein_mapping, verbose=False, throwOnMismatch=False, fileType=None):
         
     """ Infers a mapping between raw chromatogram files (mzML) and processed feature TSV files
 
@@ -428,6 +687,12 @@ def inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
     Only an excact match is allowed.
     """
     import csv, os
+
+    if fileType == "simple":
+        return simpleInferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping, sequences_mapping, protein_mapping)
+    elif fileType == "traml":
+        return tramlInferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping, sequences_mapping, protein_mapping)
+
     for file_nr, f in enumerate(aligned_pg_files):
         header_dict = {}
         if f.endswith('.gz'):
@@ -450,6 +715,8 @@ def inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
               header_dict.has_key("Charge") and \
               header_dict.has_key("aggr_Fragment_Annotation"):
                 transitions = this_row[ header_dict["aggr_Fragment_Annotation"] ].split(";")
+                if len(transitions[-1]) == 0:
+                    transitions = transitions[:-1]
                 peptide_name = this_row[header_dict["FullPeptideName"]]
                 charge_state = this_row[header_dict["Charge"]]
                 key = peptide_name + "/" + charge_state
@@ -458,6 +725,16 @@ def inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
                 mapped_precursors.append(key)
                 sequences_mapping[peptide_name] = [ key ]
 
+                if "ProteinName" in header_dict:
+                    protein_name = this_row[header_dict["ProteinName"]]
+
+                    tmp = protein_mapping.get(protein_name, [])
+                    if peptide_name not in tmp:
+                        tmp.append(peptide_name)
+                    protein_mapping[protein_name] = tmp
+
+
+
             # 1. Get the original filename (find a non-NA entry) and the corresponding run id
             if len(this_row) == 0: 
                 continue
@@ -467,6 +744,7 @@ def inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
             if aligned_id in mapping:
                 continue 
 
+
             aligned_fname = os.path.basename(this_row[ header_dict["align_origfilename"] ])
 
             # 2. Go through all chromatogram input files and try to find
